///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.AnimationDrivers;
using Fluogen.Dependz.Configuration;
using Got.Libraries;

namespace Fluogen.DeployedScripts.DeployedScripts.Abstracts.AnimationDrivers
{
	public abstract class RangeColorReducerRealFrequency : TranslateScalar<DummyScriptConfiguration>
	{
		public override void Validate( IFluonConfiguration fc, IAnimationSegmentConfiguration asc, IAnimationThreadConfiguration atc )
		{
			base.Validate( fc, asc, atc );

			if( atc.TargetValueBeginning <= 0 )
				throw new ArgumentException( $"{nameof( RangeColorReducerRealFrequency )}: atc.TargetValueBeginning <= 0" );

			if( atc.TargetValueEnd <= 0 )
				throw new ArgumentException( $"{nameof( RangeColorReducerRealFrequency )}: atc.TargetValueEnd <= 0" );

			if( atc.TargetValueBeginning.IsEqualWithinEps( atc.TargetValueEnd ) )
				throw new ArgumentException( $"{nameof( RangeColorReducerRealFrequency )}: atc.TargetValueBeginning != atc.TargetValueEnd" );
		}

		protected override void SetScalar( IFluonContext fx, IFluonConfiguration fcFrame, IAnimationThreadConfiguration atc, double progress )
		{
			GetColorReducer( fcFrame ).Frequency = progress;
		}

		protected abstract IColorReducerConfiguration GetColorReducer( IFluonConfiguration fcFrame );
	}
}
