///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.Configuration;
using Got.Libraries;

namespace Fluogen.DeployedScripts.AnimationDrivers
{
	public class ReducedColorOffset : AnimationDriver<DummyScriptConfiguration>
	{
		public override void Validate( IFluonConfiguration fc, IAnimationSegmentConfiguration asc, IAnimationThreadConfiguration atc )
		{
			base.Validate( fc, asc, atc );

			if( atc.TargetValueBeginning < 0 || atc.TargetValueBeginning > 1 )
				throw new ArgumentException( $"{nameof( ReducedColorOffset )}:"
					+ $" TargetValueBeginning < 0 || TargetValueBeginning > 1" );

			if( atc.TargetValueEnd < 0 || atc.TargetValueEnd > 1 )
				throw new ArgumentException( $"{nameof( ReducedColorOffset )}:"
					+ $" TargetValueEnd < 0 || TargetValueEnd > 1" );
		}

		public override void AffectFluonConfigurationOfFrame( IFluonContext fx, IFluonConfiguration fcSegment, IFluonConfiguration fcFrame
			, IAnimationThreadConfiguration atc, IAnimationFrameContext afx )
		{
			double progress = AnimationLib.GetProgress( afx.SegmentFrameCount, afx.SegmentFrameIndex, atc.TargetValueBeginning
				, atc.TargetValueEnd, atc.BoundariesAccelerationMode, atc.BoundariesAcceleration );

			fcFrame.ReducedColorOffset = progress;
		}

		public override Mdn AffectStratumPoint( IFluonContext fx, IAnimationThreadConfiguration atc, IAnimationFrameContext afx, Mdn x )
		{
			return x;
		}
	}
}
