///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.ColorReducers;
using Fluogen.Dependz.Configuration;
using Got.Libraries;

namespace Fluogen.DeployedScripts.ColorReducers
{
	public class LoopLnAbsXPlus1Asymptotic : AsymptoticColorReducer<DummyScriptConfiguration>
	{
		public override string Description => "x / (s2 + x); x = |rs.LoopLnAbsXPlus1( s1 )|; LoopLnAbsXPlus1 = ((1 + |x|).Ln).Loop( s1 )";

		public override void Validate( IFluonConfiguration fc, IColorReducerConfiguration crp )
		{
			base.Validate( fc, crp );

			if( crp.S1 <= 0 || !crp.S1.IsInteger() )
				throw new ArgumentException( $"{crp.Name}: crp.S1 <= 0 || !IsInteger( crp.S1 )" );

			if( crp.S2 < 1 )
				throw new ArgumentException( $"{crp.Name}: crp.S2 < 1" );
		}

		protected override double ReduceAsymptoticallyToUnitary( IFluonContext fx, IFluonConfiguration fc, IColorReducerConfiguration crp
			, double x )
		{
			int count = (int)crp.S1;

			x = x.LoopLnAbsXPlus1( count ).Abs();

			return x / (crp.S2 + x);
		}
	}
}
