///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.Configuration;
using Got.Libraries;

namespace Fluogen.DeployedScripts.ColorReducers
{
	public class OuterFadeByStratumRadiusPowAbs : ColorReducer<DummyScriptConfiguration>
	{
		public override string Description => "radius <= s1 ? rs : (rs / (1 + radius - s1) ^ s2)); radius = (sx ^ 2 + sy ^ 2).Sqrt; s1 > 0";

		public override bool SupportsSupraunitaryInputValues => false;

		public override void Validate( IFluonConfiguration fc, IColorReducerConfiguration crp )
		{
			base.Validate( fc, crp );

			if( crp.S1 <= 0 )
				throw new ArgumentException( $"{crp.Name}: crp.S1 <= 0" );
		}

		public override double ReduceToUnitary( IFluonContext fx, IFluonConfiguration fc, IPixelContext px, IColorReducerConfiguration crp
			, double x )
		{
			double radiusToStratumCenter = (px.StratumX.Sqr() + px.StratumY.Sqr()).SqrtA();

			if( radiusToStratumCenter <= crp.S1 )
				return x;

			return x / (1 + radiusToStratumCenter - crp.S1).PowA( crp.S2 );
		}
	}
}
