///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.Configuration;
using Got.Libraries;

namespace Fluogen.DeployedScripts.DomainProjectors
{
	public class StratumPeriod : DomainProjector<DummyScriptConfiguration>
	{
		public override string Description => "Stratum: (sx % s1, sy % s2)";

		public override void Validate( IFluonConfiguration fc )
		{
			base.Validate( fc );

			if( fc.DomainProjectorS1.IsZeroWithinEps() )
				throw new ArgumentException( $"{nameof( StratumPeriod )}: DomainProjectorS1 == 0" );
			if( fc.DomainProjectorS2.IsZeroWithinEps() )
				throw new ArgumentException( $"{nameof( StratumPeriod )}: DomainProjectorS2 == 0" );
		}

		public override Mdn Project( IFluonContext fx, IFluonConfiguration fc, IPixelContext px, Mdn x )
		{
			int aw = fc.AxisIndexForStratumWidth;
			int ah = fc.AxisIndexForStratumHeight;

			double rx = x[ aw ].Rest( fc.DomainProjectorS1 );
			double ry = x[ ah ].Rest( fc.DomainProjectorS2 );

			x.SetOnAxes( aw, rx, ah, ry );

			return x;
		}
	}
}
