///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.Color;
using Fluogen.Dependz.Configuration;
using Fluogen.Dependz.MdnReducers;
using Got.Libraries;

namespace Fluogen.DeployedScripts.MdnReducers
{
	public class HarshRiLnEv : FractionalIterations<DummyScriptConfiguration>
	{
		public override string Description => "ri + 1 - s2 * ev.LoopLnA( s1 )";

		public override void Validate( IFluonConfiguration fc )
		{
			base.Validate( fc );

			if( fc.MdnReducerS1 <= 0 || !fc.MdnReducerS1.IsInteger() )
				throw new ArgumentException( $"{nameof( HarshRiLnEv )}: LnA Count <= 0 || !IsInteger( LnA Count )" );
		}

		protected override double IterationFraction( IFluonContext fx, IFluonConfiguration fc, IPixelContext px, Mdn x, Mdn y, Mdn yp
			, double ev, double evp, ColorDeterminant a, int ri, bool hasEscaped )
		{
			int count = (int)fc.MdnReducerS1;
			double s2 = fc.MdnReducerS2;

			return ri + 1 - s2 * ev.LoopLnA( count );
		}
	}
}
