///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.Configuration;
using Fluogen.DeployedScripts.Libraries;

// #lib PaletteToneExtensions

namespace Fluogen.DeployedScripts.Palettes
{
	public class Lips : BitonalGradientPalette<DummyScriptConfiguration>
	{
		protected override IBitonalGradientPaletteConfiguration GetSpecificConfiguration( IFluonConfiguration fc
			, IPaletteContext paletteContext )
		{
			var configuration = base.GetSpecificConfiguration( fc, paletteContext );

			var colorModel = paletteContext.OC.ColorModel.Value;

			if( PX.OC.PaletteVariant != PaletteVariant.Specific )
			{
				configuration.PassionTone1( colorModel );
				configuration.TawnyTone2( colorModel );
			}

			return configuration;
		}

		protected override IGradientPaletteMarkList GetSpecificVariantMarks( IFluonConfiguration fc )
		{
			var colorModel = PX.OC.ColorModel.Value;

			int hue1 = PaletteToneExtensions.PassionHueHsl;
			int hue2 = PaletteToneExtensions.TawnyHueHsl;
			int edgesS = BGPC.OC.EdgesSaturation.GetValueOrDefault( 10000 );
			int edgesL = BGPC.OC.EdgesLightness.GetValueOrDefault( 0 );
			int tintS = BGPC.OC.TintSaturation.GetValueOrDefault( 10000 );
			int tintL = BGPC.OC.TintLightness.GetValueOrDefault( 10000 );

			switch( colorModel )
			{
				case ColorModelName.Hsl:
					return new BitonalGradientPaletteVariant( fc, PX, BGPC
						, new BitonalGradientPaletteToneMarkDatas
							{
								{ 0.0000, hue1, edgesS, edgesL },
								{ 0.0500, hue1,  10000,    950 },
								{ 0.2000, hue1,  10000,   2800 },
								{ 0.3500, hue1,   5300,   5400 },
								{ 0.5000, hue1,  tintS,  tintL, true },
								{ 0.6500, hue2,  10000,   6800 },
								{ 0.8000, hue2,   6800,   4600 },
								{ 0.9500, hue2,  10000,   1100 },
								{ 1.0000, hue2, edgesS, edgesL }
							}
						).ToMarkList();
				default:
					throw new InvalidOperationException( $"Unhandled color model \"{colorModel}\"" );
			}
		}
	}
}
