///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using System;
using Fluogen.Dependz;
using Fluogen.Dependz.Configuration;
using Fluogen.DeployedScripts.Libraries;

// #lib PaletteToneExtensions

namespace Fluogen.DeployedScripts.Palettes
{
	public class Lipstick : BitonalGradientPalette<DummyScriptConfiguration>
	{
		protected override IBitonalGradientPaletteConfiguration GetSpecificConfiguration( IFluonConfiguration fc
			, IPaletteContext paletteContext )
		{
			var configuration = base.GetSpecificConfiguration( fc, paletteContext );

			var colorModel = paletteContext.OC.ColorModel.Value;

			if( PX.OC.PaletteVariant != PaletteVariant.Specific )
			{
				configuration.VermilionTone1( colorModel );
				configuration.TawnyTone2( colorModel );
			}

			return configuration;
		}

		protected override IGradientPaletteMarkList GetSpecificVariantMarks( IFluonConfiguration fc )
		{
			var colorModel = PX.OC.ColorModel.Value;

			int hue1 = PaletteToneExtensions.VermilionHueHsl;
			int hue2 = PaletteToneExtensions.TawnyHueHsl;
			int edgesS = BGPC.OC.EdgesSaturation.GetValueOrDefault( 10000 );
			int edgesL = BGPC.OC.EdgesLightness.GetValueOrDefault( 10000 );
			int tintS = BGPC.OC.TintSaturation.GetValueOrDefault( 10000 );
			int tintL = BGPC.OC.TintLightness.GetValueOrDefault( 2300 );

			switch( colorModel )
			{
				case ColorModelName.Hsl:
					return new BitonalGradientPaletteVariant( fc, PX, BGPC
						, new BitonalGradientPaletteToneMarkDatas
							{
								{ 0.0000, hue1, edgesS, edgesL },
								{ 0.1500, hue1,  10000,   3500 },
								{ 0.3500, hue1,  10000,   8300 },
								{ 0.4500, hue1,  10000,   9500 },
								{ 0.5000, hue1, tintS,   tintL, true },
								{ 0.5500, hue1,  10000,   9500 },
								{ 0.6500, hue1,  10000,   8300 },
								{ 0.8500, hue2,  10000,   3800 },
								{ 1.0000, hue1, edgesS, edgesL }
							}
						).ToMarkList();
				default:
					throw new InvalidOperationException( $"Unhandled color model \"{colorModel}\"" );
			}
		}
	}
}
