///////////////////////////////////////////////////////////////////////////////
//
// Developed by Gardener of Thoughts (gardenerofthoughts.org).
// Released under a "Common Sense License".
//
///////////////////////////////////////////////////////////////////////////////

using ColorManagement.ColorModels;
using ColorManagement.Conversions;
using Fluogen.Dependz;
using Fluogen.Dependz.Color;
using Fluogen.Dependz.Configuration;

namespace Fluogen.DeployedScripts.Palettes
{
	public class TestHclabLightness : Palette<DummyScriptConfiguration>
	{
		public override int FullSize => 1 << 16;
		public override int QuickSize => 256;

		protected int H { get; } = 7600;
		protected int C { get; } = 10000;
		protected int L { get; } = 10000;

		protected override void InitializeCache( IFluonConfiguration fc )
		{
		}

		protected override PixelRgbColor ComputeColor( IFluonConfiguration fc, double scalar )
		{
			int toneHue = fc.BitonalGradientPaletteOverridingConfiguration.ToneHue1 != null
				? fc.BitonalGradientPaletteOverridingConfiguration.ToneHue1.Value : H;

			double max = TypeableColorConverter.IntegerMax;
			double h = toneHue / max;
			double c = C / max;
			double l = scalar * L / max;

			var hcl = new Hcl( h, c, l );

			var rgb = HclabConverter.ToRgb( hcl, fc.HclColorSpace, fc.BringHclInGamutOfColorSpace );

			var rgb16Bits = Rgb16Bits.FromRgbScalar( rgb );

			return PixelRgbColor.FromRgb16Bits( rgb16Bits.R, rgb16Bits.G, rgb16Bits.B );
		}
	}
}
